@@ -49,6 +49,9 @@ class KeycloakApiClient:
     def _get_users_url(self) -> str:
         return f'{self.keycloak_url}/auth/admin/realms/{self.realm}/users'
 
+    def _get_user_url(self, user_id: UUID) -> str:
+        return f'{self._get_users_url()}/{user_id}'
+
     def _get_identities_url(self, user_id: UUID) -> str:
         return f'{self._get_users_url()}/{user_id}/federated-identity'
 
@@ -72,21 +75,35 @@ class KeycloakApiClient:
                f'/protocol-mappers/models'
 
     def _get_authorization_header(self) -> str:
-        return f'Bearer {self._get_api_admin_oidc_token()}'
+        return 'Bearer ' + self._get_api_admin_oidc_token(
+            client_id=self.admin_client_id,
+            client_secret=self.admin_client_secret
+        )
+
+    def _clear_admin_user_access_token(self) -> None:
+        if self.admin_user_access_token:
+            self.admin_user_access_token = None
+
+    def _get_api_admin_oidc_token(
+        self,
+        client_id: str,
+        client_secret: Optional[str] = None
+    ) -> str:
 
-    def _get_api_admin_oidc_token(self) -> str:
         if self.admin_user_access_token:
             return self.admin_user_access_token
 
+        data = {
+            'grant_type': 'password',
+            'username': self.admin_username,
+            'password': self.admin_password,
+            'client_id': client_id
+        }
+        if client_secret:
+            data['client_secret'] = client_secret
         response = requests.post(
             self._get_token_url(),
-            data={
-                'grant_type': 'password',
-                'username': self.admin_username,
-                'password': self.admin_password,
-                'client_id': self.admin_client_id,
-                'client_secret': self.admin_client_secret
-            },
+            data=data,
             headers={'Content-Type': 'application/x-www-form-urlencoded'}
         )
 
@@ -281,18 +298,34 @@ class KeycloakApiClient:
                 federated_identities=write_keycloak_user.federated_identities
             )
 
-    def get_user_tokens(self, keycloak_id: UUID) -> KeycloakTokens:
+    def get_user_tokens(
+            self,
+            keycloak_id: UUID,
+            starting_client_id: str,
+            target_client_id: str,
+            starting_client_secret: Optional[str] = None,
+    ) -> KeycloakTokens:
+
+        self._clear_admin_user_access_token()
+
+        data = {
+            'audience': target_client_id,
+            'grant_type': 'urn:ietf:params:oauth:grant-type'
+                          ':token-exchange',
+            'requested_subject': str(keycloak_id),
+            'subject_token': self._get_api_admin_oidc_token(
+                client_id=starting_client_id,
+                client_secret=starting_client_secret
+            ),
+            'client_id': starting_client_id
+        }
+
+        if starting_client_secret:
+            data['client_secret'] = starting_client_secret
+
         response = requests.post(
             self._get_token_url(),
-            data={
-                'audience': self.token_exchange_target_client_id,
-                'grant_type': 'urn:ietf:params:oauth:grant-type'
-                              ':token-exchange',
-                'requested_subject': str(keycloak_id),
-                'subject_token': self._get_api_admin_oidc_token(),
-                'client_id': self.admin_client_id,
-                'client_secret': self.admin_client_secret,
-            },
+            data=data,
             headers={'Content-Type': 'application/x-www-form-urlencoded'}
         )
 
@@ -492,3 +525,15 @@ class KeycloakApiClient:
                 f'with ID={id_of_client}) '
                 f'response code={response.status_code}'
             )
+
+    def delete_user(self, user_id: UUID) -> None:
+        response = requests.delete(
+            url=self._get_user_url(user_id=user_id),
+            headers={'Authorization': self._get_authorization_header()}
+        )
+        if not response.ok:
+            raise KeycloakApiClientException(
+                'Error while deleting user '
+                f'with ID={user_id}) '
+                f'response code={response.status_code}'
+            )
